"""404 error handling package."""

from arduinozore.handlers.baseHandler import BaseHandler


class My404Handler(BaseHandler):
    """404 error handler."""

    def prepare(self):
        """Override prepare to cover all possible HTTP methods."""
        if self.request.protocol == 'http':
            self.redirect('https://' + self.request.host +
                          self.request.uri, permanent=False)
        self.set_status(404)
        self.render("404.html")



"""Default model package."""
import base64
import os

from arduinozore.settings import path
from yaml import SafeLoader
from yaml import YAMLObject
from yaml import dump
from yaml import safe_load


class Model(YAMLObject):
    """Model default class."""

    yaml_tag = u'!Model'
    yaml_loader = SafeLoader

    def __init__(self, name):
        """Init model."""
        self.name = name

    def __repr__(self):
        """Represent model in order to save it."""
        return "%s(name=%r)" % (self.__class__.__name__, self.name)

    @classmethod
    def load_yaml(cls, folder, filename):
        """Load yaml from file."""
        try:
            with open(path(folder, filename), 'r') as f:
                model = safe_load(f)
        except (FileExistsError, FileNotFoundError):
            model = None
        return model

    def save_yaml(self, folder):
        """Save model to file."""
        config_file = self.get_filename()
        config_file = path(folder, config_file)
        with open(config_file, 'w') as f:
            d = dump(self, default_flow_style=False,
                     allow_unicode=True, encoding=None)
            f.write(d)

    def get_filename(self):
        """Get filename to save."""
        return __class__.filenamify(self.name) + ".yaml"

    def _delete(self, folder):
        """Delete model file."""
        os.remove(path(folder, self.get_filename()))

    @classmethod
    def filenamify(cls, name):
        """Return filename base64 encoded from filename."""
        return base64.urlsafe_b64encode(name.encode('UTF-8')).decode()

    @classmethod
    def unfilenamify(cls, filename):
        """Return filename base64 decoded from filename."""
        return base64.urlsafe_b64decode(filename.encode()).decode('UTF-8')

    @classmethod
    def _get_all(cls, folder):
        """Get all models configurations."""
        models = list()
        if not os.path.exists(folder):
            os.makedirs(folder)
        config_files = os.listdir(folder)
        if config_files is not None and len(config_files) > 0:
            for config_file in config_files:
                model = cls.load_yaml(folder, config_file)
                models.append(model)
        else:
            models = None
        return models

    @classmethod
    def _get(cls, name, folder):
        """Get model by name."""
        try:
            model = cls.load_yaml(folder, cls.filenamify(name) + ".yaml")
        except Exception:
            model = None
        return model



"""Settings module."""
import os


def path(root, *a):
    """Build path from root."""
    return os.path.join(root, *a)


ROOT = os.path.dirname(
    os.path.abspath(__file__))

CONFIG_FOLDER = path(os.path.expanduser("~"), '.arduinozore')

TEMPLATE_FOLDER = path(ROOT, 'templates/')
CERT_FOLDER = path(CONFIG_FOLDER, 'certs/')
STATIC_PATH = path(CONFIG_FOLDER, 'static/')
STATIC_INSTALLER_PATH = path(ROOT, 'static_installer.sh')
CERT_INSTALLER_PATH = path(ROOT, 'install_cert.sh')

PORT = 8000
SSL_PORT = 8001

ssl_opts = {
    "certfile": path(CERT_FOLDER, "myserver.crt.pem"),
    "keyfile": path(CERT_FOLDER, "myserver.crt.key"),
}

settings = {'debug': True,
            'static_path': STATIC_PATH}
DEVICE_CONFIG_FOLDER = path(CONFIG_FOLDER, 'device')
SENSOR_CONFIG_FOLDER = path(CONFIG_FOLDER, 'sensor')
CARD_CONFIG_FOLDER = path(CONFIG_FOLDER, 'card')

ARDUINO_CODE_FILE_NAME = 'arduinozore.ino'
ARDUINO_FILE_PATH = path(ROOT, '..', 'arduino',
                         'arduinozore', ARDUINO_CODE_FILE_NAME)



"""WebSocket handler package."""
import sys

from arduinozore.handlers.serialManager import SerialManager
from tornado.escape import url_unescape
from tornado.websocket import WebSocketHandler


class WSHandler(WebSocketHandler):
    """WebSocket handler."""

    clients = []
    serial_manager = None

    def __init__(self, *args, **kwargs):
        """Init handler."""
        serial_manager = SerialManager()
        try:
            if not serial_manager.is_alive():
                serial_manager.start()
        except Exception:
            pass
        WSHandler.serial_manager = serial_manager
        super(WSHandler, self).__init__(*args, **kwargs)

    def check_origin(self, origin):
        """Check origin."""
        return True

    def open(self, slug):
        """Handle connection opening."""
        slug = url_unescape(slug)
        print('New connection was opened')
        sys.stdout.flush()
        self.port = slug
        self.write_message("Welcome to my websocket!")
        self.write_message('serial used: {}'.format(slug))
        datas = WSHandler.serial_manager.get_datas_for_port(slug)
        self.write_message(datas)
        WSHandler.clients.append(self)

    def on_message(self, message):
        """
        Handle incomming messages.

        Message only contains pin number. Thereby, toggle pin accordingly.
        """
        WSHandler.serial_manager.toggle_pin(self.port, int(message))

    def on_close(self):
        """Handle connection closing."""
        print('Connection was closed...')
        sys.stdout.flush()
        WSHandler.serial_manager.finish(self.port)
        WSHandler.clients.remove(self)

    @classmethod
    def write_to_clients(cls):
        """Send message to all clients."""
        for client in cls.clients:
            datas = WSHandler.serial_manager.get_datas_for_port(client.port)
            client.write_message(datas)
